;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Jump Maniac: An Amstrad CPC Game 
;;  Copyright (C) 2021 - Sprite Lab Enterprise (@spriteLab)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  Jump Maniac is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "../sys/globalsys.h.s"
.include "game.h.s"
.globl _game_music

key_status:: .db #1

.globl anim_counter
.globl _spr_player_0
.globl update_enemies
.globl key_map6
.globl key_map8
.globl key_map11

menu_palette:   .db HW_BLACK, HW_BRIGHT_WHITE, HW_BRIGHT_YELLOW
                .db HW_SKY_BLUE, HW_CYAN, HW_BRIGHT_CYAN, HW_BRIGHT_RED
                .db HW_MAUVE, HW_BLUE, HW_BRIGHT_GREEN, HW_ORANGE                  


endgame_palette:    .db HW_BRIGHT_WHITE, HW_BLACK, HW_MAUVE, HW_BLUE
                    .db HW_SKY_BLUE, HW_BRIGHT_GREEN, HW_BRIGHT_YELLOW
                    .db HW_ORANGE, HW_BRIGHT_RED, HW_BRIGHT_CYAN, HW_CYAN
                    .db HW_RED, HW_PINK, HW_WHITE, HW_GREEN

menu_manager::

    call cpct_akp_stop_asm

    ld de, #_game_music
    call cpct_akp_musicInit_asm
    

    ld hl, #menu_palette
    ld de, #11

    call cpct_setPalette_asm
    
    ld hl, #_screen_menu_z_end
    ld de, #0xFFFF

    call cpct_zx7b_decrunch_s_asm

    menu_loop:
        call cpct_akp_musicPlay_asm
        call cpct_waitVSYNC_asm
        halt
        halt
        
        call cpct_scanKeyboard_f_asm
        
        ;; PLAY THE GAME
        ld hl, #Key_1
        call cpct_isKeyPressed_asm
        jr nz, game_manager

        ;; SEE THE CONTROLS
        ld hl, #Key_2
        call cpct_isKeyPressed_asm
        jp nz, controls

        ;; SEE THE CREDITS
        ld hl, #Key_3
        call cpct_isKeyPressed_asm
        jp nz, credits

        jr menu_loop

    ret




game_manager:

    ;;ld de, #_game_music
    ;;call cpct_akp_musicInit_asm
    ;;call cpct_akp_musicPlay_asm
    
    call game_init  
    call game_init  
    call cpct_akp_stop_asm
    ;ld de, #_game_music
    ;call cpct_akp_musicInit_asm
    call game_loop
    
    jp menu_manager

.globl cambiomapa

game_init:
    ld a, #1
    ld (cambiomapa), a
    call key_reset
    ld a, #0
    ld (anim_counter), a
    
    ld a, #1
    ld (key_status), a
    ld ix, #player
    ld iy, #map_list
    
    ;set valores iniciales del player
    call player_set


    ld hl, #mapa_actual

    ld (hl), #<map_list
    inc hl
    ld (hl), #>map_list

    ;cambio a la game palette
    call change_palette

    ;seteo del mapa y del personaje en el mapa
    call set_map

    ld a, #0x01
    ld (map_index), a
    
    ret

key_reset:
    ld a, #1
    ld iy, #key_map6
    ld 4(iy), a
    ld iy, #key_map8
    ld 4(iy), a
    ld iy, #key_map11
    ld 4(iy), a
    ret


player_set:
    ld a, #0x00
        
    ld playervx(ix), a
    ld playervy(ix), a
    ld playervmem1(ix), a
    ld playervmem2(ix), a
    ld (gravity), a
    ld (vgravity), a
    ld a, #0xFF
    ld jump_index(ix), a
    ld a, 2(iy)
    ld playerx(ix), a
    ld (reset_pos_x), a
    
    ld a, 3(iy)
    ld playery(ix), a
    ld (reset_pos_y), a

    ld a, #3
    ld playerKeyCounter(ix), a

    ret

game_loop:

    ;call cpct_akp_musicPlay_asm

    call update_input          
    call update_physics       
    call sys_tile_collision_update
    
    
    ;call delete_enemies
    call ai_control
    ;call print_enemies
    call update_render
    call update_enemies      
    


    call cpct_waitVSYNC_asm
   
    jr    game_loop

controls:

    call cpct_akp_stop_asm
    
    ld  hl, #_screen_controls_z_end
    ld  de, #0xFFFF
    call  cpct_zx7b_decrunch_s_asm
    ld de, #_game_music
    call cpct_akp_musicInit_asm
    call exitpage
    ret

exitpage:
    call cpct_akp_musicPlay_asm
    call cpct_waitVSYNC_asm
    halt
    halt

    call cpct_scanKeyboard_f_asm
    call cpct_isAnyKeyPressed_asm
    jr z, exitpage
    jp menu_manager 


credits:

    call cpct_akp_stop_asm

    ld  hl, #_screen_credits_z_end
    ld  de, #0xFFFF
    call  cpct_zx7b_decrunch_s_asm
    ld de, #_game_music
    call cpct_akp_musicInit_asm
    call exitpage
    ret

endgame_manager::
    call cpct_scanKeyboard_f_asm
    call cpct_isAnyKeyPressed_asm
    jr z, endgame_manager
    jp menu_manager 