;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Jump Maniac: An Amstrad CPC Game 
;;  Copyright (C) 2021 - Sprite Lab Enterprise (@spriteLab)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  Jump Maniac is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "animations.h.s"
.include "globalsys.h.s"

anim_counter:: .db #0

animation_init::
    ld a, #0x00
    ld (prev_mov), a
    ret

;Devolver en hl el sprite previo
dontUpdate:
    ld h, playersprite_l(ix)
    ld l, playersprite_l+1(ix)
    dec a
    ld (anim_counter), a
    ret
    


;devuelve en hl el sprite que toque
get_player_animation::
    ld ix, #player
    ld a, (anim_counter)
    cp #0
    jr nz, dontUpdate
    ld a, #3
    ld (anim_counter), a
    ld a, (prev_mov)
    ld b, playervx(ix)
    cp b
    jr nz, obtainFirstSprite    ;si es distinto al previo, tenemos que pillar el primer sprite de la orientacion

    ;cargar pos_anim_actual si vx es -1 o 1
    ;si el valor contenido en pos_anim_actual es #0x0000, reseteamos la animacion


    ld hl, #pos_anim_actual
    ld a, (hl)
    inc hl
    ld l, (hl)
    ld h, a

    ;en hl está el puntero al array de animacion
    ;si el contenido de dicho puntero, es 0000 -> resetear animacion
    ld a, (hl)
    ld b, a
    inc hl
    ld d, (hl)
    or d        ;si el contenido de la parte izquierda y derecha es 0 --> nullptr, reset
    inc hl
    jr z, reset
    ld e, b

    ;en de está el sprite
    ;en hl está la posicion dentro del array
    ld b, h
    ld c, l
    
    ld hl, #pos_anim_actual
    
    ld (hl), b       ;  
    inc hl          ;   Guardo en pos_anim_actual, la siguiente animacion por si sigue en la misma orientacion
    ld (hl), c       ;   

    ex de, hl       ;poner en HL el sprite de DE
    
    ld playersprite_l(ix), h
    ld playersprite_l + 1(ix), l

    ret

;en b está la vx
obtainFirstSprite:
    ld a, b
    ld (prev_mov), a        ;actualizacion del movimiento previo al actual

    cp #0
    jr z, frontPos

    cp #1
    jr z, obtainRightAnim

    ;es -1
    obtainLeftAnim::
        ld h, #>animacion_izquierda
        ld l, #<animacion_izquierda
        
        ld e, (hl)
        inc hl
        ld d, (hl)
        inc hl      ;siguiente sprite

        ;HL = posicion siguiente sprite animacion
        ld b, h
        ld c, l

        ld hl, #pos_anim_actual
        ld (hl), b       ;  
        inc hl          ;   Guardo en pos_anim_actual, la siguiente animacion por si sigue en la misma orientacion
        ld (hl), c       ;   

        ex de, hl       ;Recupero el sprite en DE a HL para devolverlo

        ld playersprite_l(ix), h
        ld playersprite_l + 1(ix), l
    
    ret

;Solo es un sprite
frontPos::
    
    ld hl, #animacion_frontal
    ld e, (hl)
    inc hl
    ld d, (hl)
    inc hl      ;siguiente sprite

    ;HL = posicion siguiente sprite animacion
    ld b, h
    ld c, l

    ld hl, #pos_anim_actual
    ld (hl), b       ;  
    inc hl          ;   Guardo en pos_anim_actual, la siguiente animacion por si sigue en la misma orientacion
    ld (hl), c       ;   

    ex de, hl       ;Recupero el sprite en DE a HL para devolverlo
    
    ld playersprite_l(ix), h
    ld playersprite_l + 1(ix), l

    ret
    

obtainRightAnim::
    ld hl, #animacion_derecha
    ld e, (hl)
    inc hl
    ld d, (hl)
    inc hl      ;siguiente sprite

    ;HL = posicion siguiente sprite animacion
    ld b, h
    ld c, l

    ld hl, #pos_anim_actual
    ld (hl), b       ;  
    inc hl          ;   Guardo en pos_anim_actual, la siguiente animacion por si sigue en la misma orientacion
    ld (hl), c       ;   

    ex de, hl       ;Recupero el sprite en DE a HL para devolverlo
    
    ld playersprite_l(ix), h
    ld playersprite_l + 1(ix), l

    ret



reset:
    ld a, playervx(ix)
    cp #1
    jr z, obtainRightAnim   ;obtiene el primer sprite y reposiciona el puntero
    cp #0
    jr z, frontPos 
    jr obtainLeftAnim       





get_enemy_animation::
    ld a, (anim_counter)
    cp #0
    jr nz, noActualiza
    ld hl, #pos_enemy_actual
    ld a, (hl)
    inc hl
    ld l, (hl)
    ld h, a

    ;en hl está el puntero al array de animacion
    ;si el contenido de dicho puntero, es 0000 -> resetear animacion
    ld a, (hl)
    ld b, a
    inc hl
    ld d, (hl)
    or d        ;si el contenido de la parte izquierda y derecha es 0 --> nullptr, reset
    inc hl
    jr z, reset_enemy
    ld e, b

    ;en de está el sprite
    ;en hl está la posicion dentro del array
    ld b, h
    ld c, l
    
    ld hl, #pos_enemy_actual
    
    ld (hl), b       ;  
    inc hl          ;   Guardo en pos_anim_actual, la siguiente animacion por si sigue en la misma orientacion
    ld (hl), c       ;   

    ex de, hl       ;poner en HL el sprite de DE
    
    ld 15(ix), h
    ld 16(ix), l

    ret


reset_enemy:
    ld hl, #pos_enemy_actual
    ld (hl), #>animacion_enemigo
    inc hl
    ld (hl), #<animacion_enemigo
    
    ld hl, #_spr_enemy_0
    ld 15(ix), h
    ld 16(ix), l
    ret

noActualiza:
    ld h, 15(ix)
    ld l, 16(ix)
    ret



    