;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Jump Maniac: An Amstrad CPC Game 
;;  Copyright (C) 2021 - Sprite Lab Enterprise (@spriteLab)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  Jump Maniac is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "../man/entity.h.s"
.include "globalsys.h.s"


.include "collisions.h.s"

.globl change_map_mas_uno
.globl change_map_mas_dos
.globl change_map_menos_uno
.globl change_map_menos_dos
.globl deleteAndResetPlayer
.globl key_status
.globl player
.globl key_map6
.globl key_map8
.globl key_map11
.globl _spr_key_0
.globl draw_key
.globl end_screen
.globl exit_door
.globl _key_music
.globl _finalKey_music



sys_tile_collision_update::
    
    ld a, (map_index)
    cp #6
    ld iy, #key_map6
    call z, key_collision_update

    ld a, (map_index)
    cp #8
    ld iy, #key_map8
    call z, key_collision_update

    ld a, (map_index)
    cp #11
    ld iy, #key_map11
    call z, key_collision_update

    ld a, (map_index)
    cp #3
    call z, door_collision

    call sys_tile_collision_player_x

    ld a, (cambiomapa)
    cp #0
    ret nz

    call sys_tile_collision_player_y

    ret

entity_collision:
    
    ld e, #0
    ld ix, #player

    ld a, 0(iy)     ;x-> key_map6
    add a, 2(iy)  ;width -> key_map6
    sub a, playerx(ix)

    ret c

    ld a, playerx(ix)       ;x->player
    add a, playerwidth(ix)  ;x->width
    sub a, 0(iy)            ; x->player

    ret c

    ld a, 1(iy)             ; y->key_map6
    add a, 3(iy) ; height->key_map6
    sub a, playery(ix)      ; y->player

    ret c

    ld a, playery(ix)
    add a, playerheight(ix)
    sub a, 1(iy)

    ret c

    ld e, #1

    ret

door_collision:
    ld ix, #player
    ld a, playerKeyCounter(ix)
    cp #0
    ret nz
    ld iy, #exit_door

    call entity_collision

    ld a, e
    cp #0
    ret z

    jp nz ,end_screen

.globl reset_pos_x
.globl reset_pos_y

set_reset:
    ld a, #12
    ld (reset_pos_x), a
    ld a, #150
    ld (reset_pos_y), a
    ret

key_collision_update:

    ld a, 4(iy)
    cp #0
    ret z

    
    call entity_collision

    ld a, e
    cp #0
    ret z

    ld a, #0
    ld 4(iy), a
    ld ix, #player
    ld a, playerKeyCounter(ix)
    dec a
    ld playerKeyCounter(ix), a
    ld a, (map_index)
    cp #8
    call z, set_reset
    ld a, playerKeyCounter(ix)
    cp #0
    jr z, ultimaLlave
    ld de, #_key_music
    call cpct_akp_musicInit_asm
    ld a, #33
    
    bucleMusicaLlave:
        push af
       call cpct_akp_musicPlay_asm
       call cpct_waitVSYNC_asm
       halt
       pop af
       dec a
       cp #0
       jr nz, bucleMusicaLlave
       jr seguimosActivosPapi
    
    ultimaLlave:
    ld de, #_finalKey_music
    call cpct_akp_musicInit_asm
    ld a, #90

       bucleMusicaUltimaLlave:
       push af
       call cpct_akp_musicPlay_asm
       call cpct_waitVSYNC_asm
       halt
       pop af
       dec a
       cp #0
       jr nz, bucleMusicaUltimaLlave
       jr seguimosActivosPapi
    seguimosActivosPapi:
        call cpct_akp_stop_asm
        call draw_key   ;eliminar la llave

    ret

sys_tile_collision_player_y::
    ;; tx = x / 4
    ;; ty = y / 8
    ;; v 20 tiles de 8 de ancho
    ;; tw = tilemap width (20 en decimal, 0x14)
    ;; pos = tilemapposinic + ty * tw + tx
    
    ;; calculamos ty
    ;;ld a, e_y(ix) si cargamos directamente esta y del tilemap
    ;;estamos comprobando colisiones con la esquina superior
    ;;izquierda del personaje... como ahora queremos comprobar
    ;;colisiones con el suelo, debemos sumar la y del personaje a la
    ;; y del tilemap.
    
    ld a, playervy(ix)
    bit 7, a
    ld a, playery(ix)
    jr nz, dontUpdate
    add #(24 - 1) ;; la altura del personaje - 1 para poder detectar el suelo
    
    dontUpdate:
    
        srl a ;; |  
        srl a ;; | A = A / 8 
        srl a ;; |   
        ;; multiplicacion
        ld h, #0
        ld l, a

        add hl, hl ;; *2
        add hl, hl ;; *4
        push hl
        add hl, hl ;; *8
        add hl, hl ;; *16
        pop de
        ;; Multiplicación por 20 = *16 + *4
        add hl, de
        ld c, a ;load y value of tile
        
        ;; calculamos tx
        ld a, playerx(ix)
        ;; división entre 4
        srl a
        srl a

        
        ;hl = Ty * tw
        push hl
        call givememap  ;se carga DE y HL, devuelve en DE el puntero a mapa
        pop hl
        
        add hl, de      ;add de la direccion del mapa
        
        ;pusheamos por si hay que recalcular en la otra esquina del player
        push hl
        
        ;; calculamos tx
        ld a, playerx(ix)
        ;; división entre 4
        srl a
        srl a
        
        ld d, #0x00
        ld e, a
        add hl, de

        ;; HL = tilemapposinic + ty * tw + tx
        ld a, (hl) ;load tile number

        and #0b11100000  

        ;; Si  sale 0 es sólido
        jr z, colision_y

        ;check el pixel de la derecha
        pop hl  ;recogemos el valor de -> hl = *tile_map + ty*tw
        push hl

        ;; calculamos tx
        ld a, playerx(ix)
        inc a
        inc a
        ;; división entre 4
        srl a
        srl a

        ld d, #0x00
        ld e, a
        
        add hl, de
        ld a, (hl)

        and #0b11100000  

        ;; Si  sale 0 es sólido
        jr z, colision_y

        pop hl
        
        no_colisiona:
            
            ld a, jump_index(ix)
            cp #0xFF
            ret nz      ;si hay un salto activo y no colisiona, no hacemos nada
            
            ld a, #0x01
            ld (gravity), a
            ;ld a, (vgravity)
            ;inc a
            ;ld (vgravity), a


        ret     ;acaba la comprobacion en y

colision_y:

    ;; COMPROBACIÓN TILE COLISIÓN
    pop de

        ;; comprobación siguiente mapa | tile = 22
        ld a, (hl) ;; número de tile

        cp #21
        jp z, deleteAndResetPlayer

        cp #22
        jp z, change_map_mas_uno
        ;; comprobación anterior mapa (1 solo) | tile = 20
        cp #20
        jp z, change_map_menos_uno
        ;; comprobación siguiente mapa (2) | tile = 18
        cp #18
        jp z, change_map_mas_dos
        ;; comprobación anterior mapa (2) | tile = 28
        cp #28
        jp z, change_map_menos_dos

        ld a, playervy(ix)
        bit 7, a ;; está en la fase ascendente del salto
        jr nz, colision_y_arriba

    

    ld a, playervy(ix)
    bit 7, a
    jr nz, colision_y_arriba

    colision_y_abajo:
        
        ld a, #0x00
        
        ;si toca una plataforma, desactivamos la gravedad, y el salto, en el caso de que haya
        ld (gravity), a
        ld (vgravity), a
        ld jump_index(ix), #0xFF

        ;Reset del personaje encima de la plataforma con la que ha colisionado
        ld e, #-24
        ld d, #0
        ld l, c
        ld h, #0x00

        add hl, hl
        add hl, hl
        add hl, hl
        add hl, de      ;añadimos el tamaño del personaje a la inversa, para colocarlo justo encima del tile con el que ha colisionado

        ld playery(ix), l

        ret     ;acaba la comprobacion en y

;para que colisione por arriba tiene que haber un salto activo, en la fase ascendente
colision_y_arriba:
    
    ;activamos la gravedad
    ld a, #0x01
    ld (gravity), a
    ld a, #0x02
    ld (vgravity), a
    ;desactivamos el salto en marcha
    ld jump_index(ix), #0xFF

    ;reset del personaje donde estaba
    ld a, playery(ix)
    sub playervy(ix)
    ld playery(ix), a


    ret     ;acaba la comprobacion en y



sys_tile_collision_player_x:
    ld a, playervx(ix)
    cp #0
    ret z   ;si no se ha movido en la x, no hace falta que miremos las colisiones
    bit 7, a
    ld b, playerx(ix)
    jr nz, skipUpdatex
    inc b
    inc b
    inc b

    skipUpdatex:

        ;REORDENAR POR EFICIENCIA
    
        ld c, #0
        call checkByPlayerPosition
        ld a, e   ;en e tengo 0 o 1, dependiendo de si ha habido colision, si ya ha habido colision, no hace falta seguir mirando
        cp #0
        ;jr z, asignarColision
        ret z
        
        ld c, #6
        call checkByPlayerPosition
        ld a, e
        cp #0
        ret z

        ld c, #12
        call checkByPlayerPosition
        ld a, e
        cp #0
        ret z

        ld c, #18
        call checkByPlayerPosition
        ld a, e
        cp #0
        ret z
        
        ld c, #20
        call checkByPlayerPosition
        
        ret





;PARAMETROS
; b-> posicion en x
; c -> aumento desde la base del personaje, para checkear
checkByPlayerPosition:
    ld a, playery(ix)
    add c
    srl a ;; |  
    srl a ;; | A = A / 8 
    srl a ;; |   
    ;; multiplicacion
    ld h, #0
    ld l, a
    add hl, hl ;; *2
    add hl, hl ;; *4
    push hl
    add hl, hl ;; *8
    add hl, hl ;; *16
    pop de
    ;; Multiplicación por 20 = *16 + *4
    add hl, de
    ld a, b
    ;; calculamos tx
    ;ld a, playerx(ix)
    ;; división entre 4
    srl a
    srl a
    ;; esto es una macro que utiliza él
    ;;add__hl_a ;; hl = ty * tw + tx (faltaría sumar el tilemap)
    ld d, #0x00
    ld e, a
    add hl, de
    push hl
    call givememap
    pop hl
    add hl, de
    ;; HL = tilemapposinic + ty * tw + tx
    ld a, (hl) ;load tile number
    and #0b11100000 
    ;; Si  sale 0 es sólido
    jr z, colision_x

    ld e, #0x01

    ret


colision_x::

    ;; COMPROBACIÓN CAMBIO DE mapa_actual
    ;; HL = posición
    ;; 

    
    ;; comprobación siguiente mapa | tile = 22
    ld a, (hl) ;; número de tile
    
    cp #21
    jp z, deleteAndResetPlayer
    
    cp #22
    jp z, change_map_mas_uno
    ;; comprobación anterior mapa (1 solo) | tile = 20
    cp #20
    jp z, change_map_menos_uno
    ;; comprobación siguiente mapa (2) | tile = 18
    cp #18
    jp z, change_map_mas_dos
    ;; comprobación anterior mapa (2) | tile = 28
    cp #28
    jp z, change_map_menos_dos
    

    ld a, playerx(ix)
    sub playervx(ix)        ;reset
    ld playerx(ix), a
    ld e, #0x00
    ret


